@@ -87,7 +87,7 @@ void SpaceChildrenModel::refreshModel()
     m_rootItem =
         new SpaceTreeItem(dynamic_cast<NeoChatConnection *>(m_space->connection()), nullptr, m_space->id(), m_space->displayName(), m_space->canonicalAlias());
     endResetModel();
-    auto job = m_space->connection()->callApi<Quotient::GetSpaceHierarchyJob>(m_space->id(), Quotient::none, Quotient::none, 1);
+    auto job = m_space->connection()->callApi<Quotient::GetSpaceHierarchyJob>(m_space->id(), std::nullopt, std::nullopt, 1);
     m_currentJobs.append(job);
     connect(job, &Quotient::BaseJob::success, this, [this, job]() {
         insertChildren(job->rooms());
@@ -136,7 +136,7 @@ void SpaceChildrenModel::insertChildren(std::vector<Quotient::GetSpaceHierarchyJ
                 }
             }
             if (children[i].childrenState.size() > 0) {
-                auto job = m_space->connection()->callApi<Quotient::GetSpaceHierarchyJob>(children[i].roomId, Quotient::none, Quotient::none, 1);
+                auto job = m_space->connection()->callApi<Quotient::GetSpaceHierarchyJob>(children[i].roomId, std::nullopt, std::nullopt, 1);
                 m_currentJobs.append(job);
                 connect(job, &Quotient::BaseJob::success, this, [this, parent, insertRow, job]() {
                     insertChildren(job->rooms(), index(insertRow, 0, parent));
